import { StrictQueryParams } from '~/shared/models/strict-query.model';
import { Module, VuexModule, Mutation, Action } from 'vuex-module-decorators'

import { getFavoriteFromStorage, setFavoriteToStorage } from '~/shared/storageConsultant';

import myConsultantService from '~/shared/services/my-consultant.service';
import queryConsultantService from '~/shared/services/query-consultant.service';
import appointmentService from '~/shared/services/appointment.service';
import reviewsService from '~/shared/services/reviews.service';

import { Consultant } from '~/shared/models/consultant.model';
import { AppointmentLog } from '~/shared/models/appointment.model';
import { ClientInfo } from '~/shared/models/client.model';
import { AgentOfStrictQuery } from '~/shared/models/strict-query.model';
@Module
export default class Store extends VuexModule {
    recommendList: Consultant[] = [];
    strictQueryList: AgentOfStrictQuery[] = [];
    myConsultantList: Consultant[] = [];

    myAppointmentList: ClientInfo[] = [];
    myNewAppointmentSum: number = 0;

    myAppointmentReviewLogList: AppointmentLog[] = [];

    get isUserLogin() {
        return this.context.getters['localStorage/isUserLogin'];
    }

    @Mutation
    updateRecommend(data: Consultant[]) {
        this.recommendList = data;
    }

    @Mutation
    updateConsultantList(data: Consultant[]) {
        this.myConsultantList = data;
    }

    @Mutation
    updateStrictQueryList(data: AgentOfStrictQuery[]) {
        this.strictQueryList = data;
    }

    @Mutation
    updateMyAppointmentList(data: ClientInfo[]) {
        this.myAppointmentList = data;
    }

    @Mutation
    updateMyNewAppointmentSum(newAppointmentSum: number) {
      this.myNewAppointmentSum = newAppointmentSum;
    }

    @Mutation
    updateMyAppointmentReviewLog(data: AppointmentLog[]) {
        this.myAppointmentReviewLogList = data;
    }

    @Action
    storeRecommendList() {
        queryConsultantService.getRecommendConsultantList().then(data => {
            this.context.commit('updateRecommend', data)
        })
    }

    @Action
    async storeConsultantList() {
        const localData = getFavoriteFromStorage();

        if (!this.isUserLogin) {
            this.context.commit('updateConsultantList', localData)
            return;
        };


        if (localData?.length) {
            const agentNoList = localData.map(i => i.agentNo)
            await queryConsultantService.addFavoriteConsultant(agentNoList).then(res => {
                localStorage.removeItem('favoriteConsultant')
            })
        }

        myConsultantService.getFavoriteConsultantList().then(data => {
            this.context.commit('updateConsultantList', data);
        })
    }

    @Action
    async removeFromMyConsultantList(agentNo: string) {
        const left = this.myConsultantList.filter(item => item.agentNo !== agentNo);

        // no agent was removed
        if (left.length === this.myConsultantList.length) return false;

        if (!this.isUserLogin) {
            setFavoriteToStorage(left);
        } else {
            await myConsultantService.deleteConsultant(agentNo)
        }

        this.context.commit('updateConsultantList', left)

        return true
    }

    @Action
    async addToMyConsultantList(consultantToAdd: Consultant) {
        if (consultantToAdd) {
            const found = this.myConsultantList.find(item => item.agentNo === consultantToAdd.agentNo);
            if (!found) {
                const newData = [consultantToAdd].concat(this.myConsultantList);
                if (this.isUserLogin) {
                    await queryConsultantService.addFavoriteConsultant([consultantToAdd.agentNo])
                } else {
                    setFavoriteToStorage(newData);
                }

                this.context.commit('updateConsultantList', newData)

                return true;
            }
        }

        return false;
    }

    @Action
    storeMyAppointmentList(): void {
      appointmentService.getMyAppointmentList().then((data) => {
            const newAppointmentSum = data.filter(item => !item.consultantViewTime || item.consultantViewTime === null).length;
            this.context.commit('updateMyAppointmentList', data);
            this.context.commit('updateMyNewAppointmentSum', newAppointmentSum);
        });
    }

    @Action
    storeMyAppointmentReviewLog() {
        reviewsService.getMyReviewLog().then((data) => {
            const dataWithLatestDate = data.map((item) => {
                return {
                    ...item,
                    compareDate: new Date(item.lastModifiedDate)
                }
            });
            const sortedData = dataWithLatestDate.sort((a, b) => +b.compareDate - +a.compareDate);
            this.context.commit('updateMyAppointmentReviewLog', sortedData);
        });
    }

    @Action
    updateMyAppointment(myAppointment: ClientInfo) {
        const data = this.myAppointmentList.filter(item => item.id !== myAppointment.id);
        data.unshift(myAppointment);
        this.context.commit('updateMyAppointmentList', data)
    }

    @Action
    async storeStrictQueryList(strictQueryDto: StrictQueryParams) {
        return await queryConsultantService.strictQuery(strictQueryDto).then(res=>{
            this.context.commit('localStorage/storageRecommendConsultant', JSON.stringify(strictQueryDto));
            this.context.commit('updateStrictQueryList', res)
            return res.length;
        });
    }

}
